"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
Autogenerated by poetry-setup:
https://github.com/orsinium/poetry-setup
"""
# IMPORTANT: this file is autogenerated. Do not edit it manually.
# All changes will be lost after `poetry-setup` command execution.
# ----------------------------------------------------------------
# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from os import path
# io.open is needed for projects that support Python 2.7
# It ensures open() defaults to text mode with universal newlines,
# and accepts an argument to specify the text encoding
# Python 3 only projects can skip this import
from io import open
here = path.abspath(path.dirname(__file__))
# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()
# Arguments marked as "Required" below must be included for upload to PyPI.
# Fields marked as "Optional" may be commented out.
setup(
    # https://packaging.python.org/specifications/core-metadata/#name
    name='pillars',  # Required
    # https://www.python.org/dev/peps/pep-0440/
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.1.1',  # Required
    # https://packaging.python.org/specifications/core-metadata/#summary
    description="",  # Required
    # https://packaging.python.org/specifications/core-metadata/#description-optional
    long_description=long_description,  # Optional
    # https://packaging.python.org/specifications/core-metadata/#description-content-type-optional
    long_description_content_type='text/markdown',  # Optional (see note above)
    author="Allocloud",  # Optional
    author_email="allocloud@ovv.wtf",  # Optional
    # For a list of valid classifiers, see https://pypi.org/classifiers/
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7'
    ],  # Optional
    packages=find_packages(),  # Required
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=[
        'cython (>=0.28.5,<0.29.0)',
        'uvloop (>=0.11.2,<0.12.0)',
        'cchardet (>=2.1,<3.0)',
        'aiohttp (>=3.3,<4.0)',
        'asyncpg (>=0.17.0,<0.18.0)',
        'attrs (>=18.1,<19.0)',
        'ujson (>=1.35,<2.0)',
        'async-timeout (>=3.0,<4.0)',
        'setproctitle (>=1.1,<2.0)',
        'aiodns (>=1.1,<2.0)',
        'pyyaml (>=3.13,<4.0)',
        'aioredis (>=1.1,<2.0)',
        'cerberus (>=1.2,<2.0)',
    ],  # Optional
    # https://setuptools.readthedocs.io/en/latest/setuptools.html#dependencies-that-aren-t-in-pypi
    dependency_links=[
        'git+https://github.com/eyepea/aiosip.git@master#egg=aiosip',
        'git+https://github.com/gawel/panoramisk.git@master#egg=panoramisk',
    ],  # Optional
    # https://stackoverflow.com/a/16576850
    include_package_data=True,
    # https://packaging.python.org/specifications/core-metadata/#project-url-multiple-use
    project_urls={  # Optional
    },
)



import logging
from typing import Awaitable, Callable, Optional

import aiohttp.web
import cerberus
from aiohttp.web_urldispatcher import UrlDispatcher
import ujson

from ..exceptions import DataValidationError
from ..request import BaseRequest, Response

LOG = logging.getLogger(__name__)


@aiohttp.web.middleware
async def middleware(
    request: aiohttp.web.Request,
    handler: Callable[["HttpRequest"], Awaitable[aiohttp.web.Response]],
):
    common_request = HttpRequest(request)
    response = await handler(common_request)
    if isinstance(response, Response):
        return aiohttp.web.json_response(status=response.status, data=response.data)
    else:
        return response


class HttpRequest(BaseRequest):
    def __init__(self, request: aiohttp.web.Request) -> None:
        super().__init__(request.app.state)
        self._request = request
        self._data: Optional[dict] = None
        self["validator"] = self._request["validator"]

    async def data(self, validate: bool = None) -> dict:
        if self._data is None:
            if "json" in self["config"]:
                self._data = await self._request.json(loads=ujson.loads)
            elif self._request.method == "GET":
                self._data = dict(self._request.query)
            else:
                self._data = {"text": await self._request.text()}
                return self._data

            # TODO mypy (self._data can not be None)
            self._data.update(self._request.match_info)  # type: ignore

            if validate:
                if self["validator"].validate(self._data):
                    self._data = self["validator"].document
                else:
                    raise DataValidationError(self["validator"].errors)

        return self._data or dict()

    @property
    def initial(self) -> aiohttp.web.Request:
        return self._request

    @property
    def config(self) -> dict:
        return self._request["config"]

    @property
    def method(self) -> str:
        return self._request.method

    @property
    def path(self) -> str:
        return self._request.path


class Application(aiohttp.web.Application):
    def __init__(self, **kwargs) -> None:
        if "router" not in kwargs:
            kwargs["router"] = Router()

        if "middlewares" not in kwargs:
            kwargs["middlewares"] = (middleware,)
        else:
            kwargs["middlewares"] = list(kwargs["middlewares"])
            kwargs["middlewares"].insert(0, middleware)

        super().__init__(**kwargs)


class Router(UrlDispatcher):
    def __init__(self) -> None:
        super().__init__()
        self.config: dict = dict()
        self.validators: dict = dict()

    def add_route(
        self,
        *args,
        config: Optional[dict] = None,
        data_schema: Optional[dict] = None,
        **kwargs
    ):
        route = super().add_route(*args, **kwargs)

        if data_schema:
            self.validators[route] = cerberus.Validator(data_schema)

        if config:
            self.config[route] = set(config)
        else:
            self.config[route] = set()
        return route

    async def resolve(self, request: aiohttp.web.Request) -> aiohttp.web.Resource:
        resource = await super().resolve(request)
        request["config"] = self.config.get(resource.route, ())
        request["validator"] = self.validators.get(resource.route)
        return resource
